// Advanced application for feeding data to Pachube. I used 3 thermometers, Photodiode, Sound level detector, PIR.
// Based on pachube.pde 2011-07-08 <jcw@equi4.com> http://opensource.org/licenses/mit-license.php
// Originally written for Nanode at http://maniacbug.wordpress.com/2011/08/07/nanode/
// Need EtherCard drivers http://jeelabs.net/projects/11/wiki/EtherCard
// Schematics like in Quick And Dirty Arduino Ethernet http://www.komputer.de/wordpress/archives/497
// Except ENC28J60 CS signal is not connected to Dig10, but connected to Dig8 like in Nanode. 


#include <EtherCard.h>

#define FEED "44959"
#define APIKEY "qpLG77lHBQVhhJlJ5yAhOAin_CggAaW5tnTVnyGj09k"

#include <OneWire.h>
#include <DallasTemperature.h>
#define TEMPERATURE_PRECISION 12
#define ONE_WIRE_BUS 4 // Onewire data wire is plugged into port 2 on the Arduino. Needs ca 4k pullup resistor to 5V.

OneWire oneWire(ONE_WIRE_BUS); // Setup 1Wire instance to communicate with any OneWire devices (not just Maxim/Dallas temperature ICs)
DallasTemperature sensors(&oneWire); // Pass our oneWire reference to Dallas Temperature. 
DeviceAddress Thermometer1, Thermometer2, Thermometer3; // arrays to hold device addresses

int count=0; //byte count in serial message
int n=0, N=300;
int a0=0, a1=0, a2=0, a3=0, c0=0, c1=0, c2=0, c3=0;  //  analog input read values, averaged analog input values
long int b0=0, b1=0, b2=0, b3=0; // sums over 300 seconds

byte mymac[] = { 0x74,0x69,0x69,0x2D,0x30,0x31 };

static byte static_ip[] = { 192,168,178,200 };
static byte static_gw[] = { 192,168,178,1 };
static byte static_dns[] = { 194,25,0,52 };

char website[] PROGMEM = "api.pachube.com";

byte Ethernet::buffer[500];
uint32_t timer;
Stash stash;

void printf_begin(void);
void read_MAC(byte*);

void setup () {
  Serial.begin(9600);
  Serial.println ("AI0-AI3 1w@pin4 R=4.7k 300s avg");
   sensors.begin();
    if (!sensors.getAddress(Thermometer1, 0)) Serial.println("miss1"); 
    if (!sensors.getAddress(Thermometer2, 1)) Serial.println("miss2"); 
    if (!sensors.getAddress(Thermometer3, 2)) Serial.println("miss3"); 
  sensors.setResolution(Thermometer1, TEMPERATURE_PRECISION); sensors.setResolution(Thermometer2, TEMPERATURE_PRECISION);
  sensors.setResolution(Thermometer3, TEMPERATURE_PRECISION); 


  printf_begin();
  printf_P(PSTR("\nEtherCard/examples/pachube\n\r"));

  printf_P(PSTR("MAC: %02x:%02x:%02x:%02x:%02x:%02x\n\r"),
      mymac[0],
      mymac[1],
      mymac[2],
      mymac[3],
      mymac[4],
      mymac[5]
  );

  if (ether.begin(sizeof Ethernet::buffer, mymac) == 0)
    printf_P(PSTR( "Failed to access Ethernet controller\n\r"));

    ether.staticSetup(static_ip, static_gw);
    ether.copyIp(ether.dnsip, static_dns);

  ether.printIp("IP: ", ether.myip);
  ether.printIp("GW: ", ether.gwip);
  ether.printIp("DNS: ", ether.dnsip);

  if (!ether.dnsLookup(website))
    printf_P(PSTR("DNS failed\n\r"));
    
  ether.printIp("SRV: ", ether.hisip);
}


void loop () {
  ether.packetLoop(ether.packetReceive());
  
a0=analogRead(A0); if (a0<100){a0=0;} if (a0>110){a0=1;} a1=analogRead(A1);a2=analogRead(A2);a3=analogRead(A3);
b0=b0+a0; b1=b1+a1; b2=b2+a2, b3=b3+a3;
n++;
if(n==N){c0=b0; c1=b1/N; c2=b2/N; c3=b3/N; b0=0; b1=0; b2=0; b3=0; n=0;}

sensors.requestTemperatures();  
float t1 = sensors.getTempC(Thermometer1); Serial.print(t1); Serial.print(" ");
float t2 = sensors.getTempC(Thermometer2); Serial.print(t2); Serial.print(" ");
float t3 = sensors.getTempC(Thermometer3); Serial.print(t3); Serial.print("   ");

Serial.print(a0);Serial.print(" ");Serial.print(a1);Serial.print(" ");Serial.print(a2);Serial.print(" ");Serial.print(a3);Serial.print("    "); 
Serial.print(c0);Serial.print(" ");Serial.print(c1);Serial.print(" ");Serial.print(c2);Serial.print(" ");;Serial.print(c3);Serial.println(" "); 
delay(166);  
    
  
  if (millis() > timer ) {
    timer = millis() + 60000;

    printf_P(PSTR("Sending...\n\r"));
    
    // generate two fake values as payload - by using a separate stash,
    // we can determine the size of the generated message ahead of time
    byte sd = stash.create();
    stash.print("1,");
    stash.println(t1);
    stash.print("2,");
    stash.println(t2);
    stash.print("3,");
    stash.println(t3);

    stash.print("4,");
    stash.println(c0);
    stash.print("5,");
    stash.println(c1);
    stash.print("6,");
    stash.println(c2);
    stash.print("7,");
    stash.println(c3);

    stash.save();
    
    // generate the header with payload - note that the stash size is used,
    // and that a "stash descriptor" is passed in as argument using "$H"
    Stash::prepare(PSTR("PUT http://$F/v2/feeds/$F.csv HTTP/1.0" "\r\n"
                        "Host: $F" "\r\n"
                        "X-PachubeApiKey: $F" "\r\n"
                        "Content-Length: $D" "\r\n"
                        "\r\n"
                        "$H"),
            website, PSTR(FEED), website, PSTR(APIKEY), stash.size(), sd);

    // send the packet - this also releases all stash buffers once done
    ether.tcpSend();
  }
}

int serial_putc( char c, FILE * )
{
  Serial.write( c );
  return c;
}

void printf_begin(void)
{
  fdevopen( &serial_putc, 0 );
}

