// Simple demo for feeding some data to Pachube.
// Based on pachube.pde 2011-07-08 <jcw@equi4.com> http://opensource.org/licenses/mit-license.php
// Originally written for Nanode at http://maniacbug.wordpress.com/2011/08/07/nanode/
// Need EtherCard drivers http://jeelabs.net/projects/11/wiki/EtherCard
// Schematics like in Quick And Dirty Arduino Ethernet http://www.komputer.de/wordpress/archives/497
// Except ENC28J60 CS signal is not connected to Dig10, but connected to Dig8 like in Nanode. 

#include <EtherCard.h>

#define FEED "44959"
#define APIKEY "qpLG77lHBQVhhJlJ5yAhOAin_CggAaW5tnTVnyGj09k"

byte mymac[] = { 0x74,0x69,0x69,0x2D,0x30,0x31 }; // without MAC it will not work

static byte static_ip[] = { 192,168,178,200 };
static byte static_gw[] = { 192,168,178,1 };
static byte static_dns[] = { 194,25,0,52 };

char website[] PROGMEM = "api.pachube.com";

byte Ethernet::buffer[500];
uint32_t timer;
Stash stash;

void printf_begin(void);
void read_MAC(byte*);

void setup () {
  Serial.begin(9600);
  printf_begin();
  printf_P(PSTR("\nEtherCard/examples/pachube\n\r"));

  printf_P(PSTR("MAC: %02x:%02x:%02x:%02x:%02x:%02x\n\r"),
      mymac[0],
      mymac[1],
      mymac[2],
      mymac[3],
      mymac[4],
      mymac[5]
  );

  if (ether.begin(sizeof Ethernet::buffer, mymac) == 0)
    printf_P(PSTR( "Failed to access Ethernet controller\n\r"));

    ether.staticSetup(static_ip, static_gw);
    ether.copyIp(ether.dnsip, static_dns);

  ether.printIp("IP:  ", ether.myip);
  ether.printIp("GW:  ", ether.gwip);
  ether.printIp("DNS: ", ether.dnsip);

  if (!ether.dnsLookup(website))
    printf_P(PSTR("DNS failed\n\r"));
    
  ether.printIp("SRV: ", ether.hisip);
}

void loop () {
  ether.packetLoop(ether.packetReceive());
  
  if (millis() > timer ) {
    timer = millis() + 10000;

    printf_P(PSTR("Sending...\n\r"));
    
    // generate two fake values as payload - by using a separate stash,
    // we can determine the size of the generated message ahead of time
    byte sd = stash.create();
    stash.print("1,");
    stash.println(123);
    stash.print("2,");
    stash.println(456);
    stash.save();
    
    // generate the header with payload - note that the stash size is used,
    // and that a "stash descriptor" is passed in as argument using "$H"
    Stash::prepare(PSTR("PUT http://$F/v2/feeds/$F.csv HTTP/1.0" "\r\n"
                        "Host: $F" "\r\n"
                        "X-PachubeApiKey: $F" "\r\n"
                        "Content-Length: $D" "\r\n"
                        "\r\n"
                        "$H"),
            website, PSTR(FEED), website, PSTR(APIKEY), stash.size(), sd);

    // send the packet - this also releases all stash buffers once done
    ether.tcpSend();
  }
}

int serial_putc( char c, FILE * )
{
  Serial.write( c );
  return c;
}

void printf_begin(void)
{
  fdevopen( &serial_putc, 0 );
}
